<?php
/**
 * MVClasses
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * @category    MVClasses
 * @package     Controler
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */

/**
 * @see FilterChain
 */
require_once dirname(__FILE__)."/../Filter/FilterChain.php";
/**
 * Registry
 */
require_once dirname(__FILE__)."/../Registry/Registry.php";

/**
 * Enter description here...
 *
 * Longdesc
 *
 * @category    MVClasses
 * @package     Controler
 * @copyright 	Copyright (c) 2007 Sven Strittmatter
 * @author	"Sven Strittmatter" <ausserirdisch@sven-space.de>
 * @version 	1.0
 * @link        http://www.sven-space.de/code/php/MVClasses
 */
class FrontController {
    /**
     * Haelt die Referenz auf ein ComamndResolver-Objekt.
     *
     * @access	private
     * @var		CommandResolver
     */
    private $_resolver;
    /**
     * Haelt eine Referenz auf ein FilterChain-Objekt,
     * welches Filter-Objekte beinhalten kann, die vor
     * dem eigentlichen Command ausgef�hrt werden.
     *
     * @access 	private
     * @var 	FilterChain
     */
    private $_pre_filter_chain;
    /**
     * Haelt eine Referenz auf ein FilterChain-Objekt,
     * welches Filter-Objekte beinhalten kann, die nach
     * dem eigentlichen Command ausgef�hrt werden.
     *
     * @access 	private
     * @var 	FilterChain
     */
    private $_post_filter_chain;

    /**
     * Konstruktor
     *
     * @access 	public
     * @param	CommandResolver $resolver
     */
    public function __construct(CommandResolver $resolver) {
        $this->_resolver = $resolver;
        $this->_pre_filter_chain = new FilterChain();
        $this->_post_filter_chain = new FilterChain();
    }

    /**
     * Handled den Clientrequest. Es mu� dass Request- und
     * Response-Objekt uebergeben werden, an welchen die
     * Filterchains und das geforderte Command durchgefuehrt
     * wird
     *
     * @access 	public
     * @param	Request $req
     * @param	Response $res
     */
    public function handleRequest(Request $req, Response $res) {
        $reg = Registry::getInstance();
        $reg->setRequest($req);
        $reg->setResponse($res);
        /*
         * Alle Filter in der FilterChain $_pre_filter_chain
         * werden angewendet.
         */
        $this->_pre_filter_chain->processFilters($req, $res);
        /*
         * Das vom CommandResolver zurueckgegebene Command
         * wird ausgefuehrt.
         */
        $command = $this->_resolver->getCommand($req);
        $command->execute($req, $res);
        /*
         * Alle Filter in der FilterChain $_post_filter_chain
         * werden angewendet.
         */
        $this->_post_filter_chain->processFilters($req, $res);
        /*
         * Die Ausgabe wird an den Client gesendet.
         */
        $res->flush();
    }

    /**
     * Fuegt einen Filterobjekt der FilterChain hinzu, welches
     * vor dem eigentlich Command ausgefuehrt wird.
     *
     * @access 	public
     * @param 	Filter $f
     */
    public function addPrefilter(Filter $f) {
        $this->_pre_filter_chain->addFilter($f);
    }

    /**
     * Fuegt einen Filterobjekt der FilterChain hinzu, welches
     * nach dem eigentlich Command ausgefuehrt wird.
     *
     * @access 	public
     * @param 	Filter $f
     */
    public function addPostFilter(Filter $f) {
        $this->_post_filter_chain->addFilter($f);
    }
}